%--------------------------------------------------------------------------
%--------------------------------------------------------------------------                       
%                EVOLUTIONARY ANNEALING-SIMPLEX ALGORITHM
%--------------------------------------------------------------------------
%--------------------------------------------------------------------------
% Developed by: Kossieris Panagiotis and Tsoukalas Ioannis 
% 
% Based on the publication of:
%
% Efstratiadis, A., and D. Koutsoyiannis, 
% An evolutionary annealing-simplex algorithm for global optimisation of water resource systems, 
% Proceedings of the Fifth International Conference on Hydroinformatics, 
% Cardiff, UK, 14231428, International Water Association, 2002.
%
% More: http://www.itia.ntua.gr/el/docinfo/524/
%--------------------------------------------------------------------------
%--------------------------------------------------------------------------
% SYNTAX:
%
% [BestValue,BestPar,MoreResults] = EAS(n,m,xmin,xmax,fn,maxeval,beta,maxclimbs,pmut,ratio,ftol)
%
%                                INPUT ARGUMENTS
%
%  n : problem dimension
%  m : population count ( m > n+1 )
%  xmin : lower parameter interior bounds
%  xmax : upper parameter interior bounds
%  xlow : lower parameter exterior bounds
%  xup : upper parameter exterior bounds
%  fn : objective function
%  maxeval : maximum number of function evaluations
%  ftol : the fractional convergence tolerance to be achieved in the function value for an early return
%  ratio : fraction of temperature reduction, when a local minimum is found
%  pmut : probability of accepting an offspring generated via mutation
%  beta : annealing schedule parameter
%  maxclimbs : maximum number of uphill steps
%
%                               OUTPUT ARGUMENTS
%
%   BestValue : the minimal value of the objective function
%   BestPar : the optimal values of control variables
%
%   MATLAB structure (MoreResults) with the following elements:
%     - MoreResults.NumIter : number of iterations taken by algorithm
%     - MoreResults.Ftolpop : fractional convergence tolerance of population generated at the last iteration
%     - MoreResults.fmin: the minimum value of every iteration
%     - MoreResults.fperiter: cumulative function evaluation counter per iteration
%     - MoreResults.S: archive of parameters (decision variables) employed by the algorithm
%     - MoreResults.Y: archive of objective function values for parameters stored in S
%     - MoreResults.Temp: The temperature of annealing process
%--------------------------------------------------------------------------
%--------------------------------------------------------------------------
% USAGE:
%
% Ackley function example

% with 6 input arguments
% xmin=ones(1,10)*-5.12; xmax=ones(1,10)*5.12;maxeval=8000;
% [BestValue,BestPar,MoreResults] = EAS(10,50,xmin,xmax,@ackley,maxeval);

% with 7 input arguments
% xmin=ones(1,10)*-5.12; xmax=ones(1,10)*5.12;maxeval=8000;
% [BestValue,BestPar,MoreResults] = EAS(10,50,xmin,xmax,@ackley,Maxeval,3)

% with 8 input arguments
% xmin=ones(1,10)*-5.12; xmax=ones(1,10)*5.12;maxeval=8000;
% [BestValue,BestPar,MoreResults] = EAS(10,50,xmin,xmax,@ackley,Maxeval,3,4)

% with 9 input arguments
% xmin=ones(1,10)*-5.12; xmax=ones(1,10)*5.12;maxeval=8000;
% [BestValue,BestPar,MoreResults] = EAS(10,50,xmin,xmax,@ackley,Maxeval,3,4,0.1)

% with 10 input arguments
% xmin=ones(1,10)*-5.12; xmax=ones(1,10)*5.12;maxeval=8000;
% [BestValue,BestPar,MoreResults] = EAS(10,50,xmin,xmax,@ackley,Maxeval,3,4,0.1,0.95)

% with 11 input arguments
% xmin=ones(1,10)*-5.12; xmax=ones(1,10)*5.12;maxeval=8000;
% [BestValue,BestPar,MoreResults] = EAS(10,50,xmin,xmax,@ackley,Maxeval,3,4,0.1,0.95,10^-6)

% When a parameter is ommited the following default parameters are used:
% beta=3; % maxclimbs=3; % pmut=0.1; % ratio=0.95; % ftol=10^-6;